#include "WMSLayerRenderer.h"

#include "WMSLayer.h"
#include "Utils.h"

#include "../../../../core/translator/Translator.h"
#include "../../../../maptools/AbstractLayer.h"
#include "../../../../maptools/Canvas.h"
#include "../../../../maptools/Utils.h"
#include "../../../../dataaccess/utils/Utils.h"
#include "../../../../geometry/Envelope.h"
#include "../../../../se/Utils.h"
#include "../../../../se/CoverageStyle.h"
#include "../../../../raster/RasterProperty.h"
#include "../../../core/Exception.h"
#include "../../../core/Utils.h"
#include "../../../../srs/Config.h"
#include "../../../../srs/SpatialReferenceSystemManager.h"

// Boost
#include <boost/format.hpp>

te::ws::ogc::wms::WMSLayerRenderer::WMSLayerRenderer()
{

}

te::ws::ogc::wms::WMSLayerRenderer::~WMSLayerRenderer()
{

}

void te::ws::ogc::wms::WMSLayerRenderer::draw(te::map::AbstractLayer *layer, te::map::Canvas *canvas, const te::gm::Envelope &bbox, int srid, const double &scale, bool *cancel)
{

  te::ws::ogc::wms::WMSLayer* wmsLayer = dynamic_cast<te::ws::ogc::wms::WMSLayer*>(layer);

  if(wmsLayer == nullptr)
  {
    throw te::ws::core::Exception() << te::ErrorDescription(TE_TR("Wrong type render type for this layer!"));
  }

  te::gm::Envelope reprojectedBBOX(bbox);

  if(srid != TE_UNKNOWN_SRS)
  {
    reprojectedBBOX.transform(srid, wmsLayer->getSRID());
  }
  else if(layer->getSRID() != srid)
  {
    throw te::ws::core::Exception() << te::ErrorDescription(TE_TR("The layer or map has no valid SRID!"));
  }

  if(!reprojectedBBOX.intersects(wmsLayer->getExtent()))
    return;

  te::rst::Raster* raster = te::ws::ogc::wms::GetMapRaster(wmsLayer, srid, bbox, canvas->getWidth(), canvas->getHeight());

  wmsLayer->setCurrentRaster(raster);

  std::unique_ptr<te::da::DataSet> dataSet = wmsLayer->getData();

  std::unique_ptr<te::da::DataSetType> dsType = wmsLayer->getDataSetType();

  te::rst::RasterProperty* rasterProperty = te::da::GetFirstRasterProperty(dsType.get());

  // get the associated layer style
  te::se::Style* style = wmsLayer->getStyle();
  if(style == 0)
  {
    // try create an appropriate style
    style = te::se::CreateCoverageStyle(rasterProperty->getBandProperties());

    if(style == 0)
      throw te::ws::core::Exception() << te::ErrorDescription((boost::format(TE_TR("Could not create a default coverage style to the layer %1%.")) % wmsLayer->getTitle()).str());

    wmsLayer->setStyle(style);
  }

  // should I render this style?
  te::se::CoverageStyle* cs = dynamic_cast<te::se::CoverageStyle*>(style);
  if(cs == 0)
    throw te::ws::core::Exception() << te::ErrorDescription(TE_TR("The layer style is not a Coverage Style!"));

  std::unique_ptr<te::rst::Raster> memRaster (te::map::GetRaster(wmsLayer));

  te::map::DrawRaster(memRaster.get(), canvas, bbox, srid, bbox, srid, cs, layer->getRasterContrast(), scale, cancel);
}
